package com.yiqi.order.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConfig;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.request.AlipayTradePayRequest;
import com.alipay.api.request.AlipayTradeQueryRequest;
import com.alipay.api.request.AlipayTradeRefundRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.alipay.api.response.AlipayTradePayResponse;
import com.alipay.api.response.AlipayTradeQueryResponse;
import com.alipay.api.response.AlipayTradeRefundResponse;
import com.github.wxpay.sdk.WXPay;
import com.github.wxpay.sdk.WXPayUtil;
import com.wechat.pay.java.service.payments.app.AppServiceExtension;
import com.wechat.pay.java.service.payments.app.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.model.Payer;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.yiqi.common.config.AlipayConfigs;
import com.yiqi.common.enums.*;
import com.yiqi.common.exception.ServiceException;
import com.yiqi.common.config.WxMaProperties;
import com.yiqi.common.config.WxPayConfigs;
import com.yiqi.common.core.domain.AuthHolder;
import com.yiqi.common.core.domain.R;
import com.yiqi.common.utils.DateUtils;
import com.yiqi.common.utils.EnumUtils;
import com.yiqi.common.utils.OrderNoUtils;
import com.yiqi.common.utils.StringUtils;
import com.yiqi.common.utils.pay.WxPayV2Config;
import com.yiqi.order.domain.OrderPayInfo;
import com.yiqi.order.domain.dto.*;
import com.yiqi.order.domain.vo.weapp.AppPayRespVO;
import com.yiqi.order.service.*;
import com.yiqi.payment.AliPayService;
import com.yiqi.payment.WechatPayService;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;


/**
 * @author xiaohei
 * @date 2023/12/25
 */
@Service
public class PayServiceImpl implements IPayService {
    @Autowired
    private ScheduledExecutorService scheduledExecutorService;
    @Autowired
    private OrderNoUtils orderNoUtils;
    @Autowired
    private IOrderRechargeService orderRechargeService;
    @Autowired
    private IOrderActivityService orderActivityService;
    @Autowired
    private IOrderGoodsService orderGoodsService;
    @Autowired
    private IOrderClothService orderClothService;
    @Autowired
    private WxMaProperties wxMaProperties;
    @Autowired
    private AliPayService aliPayService;
    @Autowired
    private WechatPayService wechatPayService;
    @Autowired
    private IOrderPayInfoService orderPayInfoService;

    private static final Logger log = LoggerFactory.getLogger(PayServiceImpl.class);


    @Override
    @Transactional(rollbackFor = Exception.class)
    public R unifiePaySuccessNotify(String orderPayNo, String orderNo, String outTradeNo) {
        String header = orderPayNo.substring(0, 2);
        if (StringUtils.isBlank(orderNo)) {
            OrderPayInfo orderPayInfo = orderPayInfoService.getOrderPayInfoByPayOrderNo(orderPayNo);
            if (orderPayInfo == null) {
                return R.fail("未查询到订单");
            }
            orderNo = orderPayInfo.getOrderNo();
        }

        switch (EnumUtils.getEnumByType(header, OrderNoHeaderType.class)) {
            case cloth_order_header: {
                return orderClothService.paySuccess(orderNo, outTradeNo);
            }
            case goods_order_header: {
                return orderGoodsService.handlePaySuccessUpdateOrderStatus(orderNo);
            }
            case recharge_order_header: {
                return orderRechargeService.creatPassWordCardByPayNotify(orderNo, outTradeNo);
            }
            case activity_order_header: {
                return orderActivityService.handlePaySuccessUpdateActivityOrderStatus(orderNo);
            }
        }
        return R.fail("订单类型不正确");
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public R unifieRefundSuccessNotify(String refundNo, String orderNo, String outTradeNo) {
        String header = refundNo.substring(0, 2);
        switch (EnumUtils.getEnumByType(header, OrderNoHeaderType.class)) {
            case cloth_order_header: {
                return orderClothService.refundSuccess(refundNo, orderNo);
            }
            case goods_order_header: {
                return orderGoodsService.handlePaySuccessUpdateOrderStatus(orderNo);
            }
            case recharge_order_header: {
                orderRechargeService.refundSuccess(orderNo);
                break;
            }
            case activity_order_header: {
                return orderActivityService.handlePaySuccessUpdateActivityOrderStatus(orderNo);
            }
        }
        return R.fail("订单类型不正确");
    }


    @Override
    public R alipayPayExpireOrCancelNotify(String orderNo) {
        String header = orderNo.substring(0, 2);
        switch (EnumUtils.getEnumByType(header, OrderNoHeaderType.class)) {
            case cloth_order_header: {
                break;
            }
            case goods_order_header: {
                orderGoodsService.handlePayFailCancelOrder(orderNo);
                break;
            }
            case recharge_order_header: {
                orderRechargeService.handleCancelRecharge(orderNo);
                break;
            }
            case activity_order_header: {
                orderActivityService.handleCancelActivityOrder(orderNo);
                break;
            }
        }
        return R.fail("订单类型不正确");
    }

    @Override
    public AppPayRespVO appPay(PayOrderAppDTO payOrderAppDTO) {
        switch (EnumUtils.getEnumByType(payOrderAppDTO.getPayType(), PayType.class)) {
            case WECHAT: {
                if ("APP".equals(payOrderAppDTO.getPlatform())) {
                    return wechatPayService.appPay(payOrderAppDTO);
                } else {
                    payOrderAppDTO.setUserId(AuthHolder.openId());
                    return wechatPayService.xcxPay(payOrderAppDTO);
                }
            }
            case ALIPAY: {
                return aliPayService.appPay(payOrderAppDTO);
            }
            default:
                throw new ServiceException("支付方式为" + EnumUtils.getEnumByType(payOrderAppDTO.getPayType()
                        , PayType.class).getInfo() + "不支持第三方APP支付");
        }
    }

    /**
     * 退款
     *
     * @param payRefundDTO
     * @return 退款状态 OrderRefundStatus
     */
    @Override
    public String refund(PayRefundDTO payRefundDTO) {
        switch (EnumUtils.getEnumByType(payRefundDTO.getPayType(), PayType.class)) {
            case WECHAT: {
                return wxRefund(payRefundDTO);
            }
            case ALIPAY: {
                return alipayRefund(payRefundDTO);
            }
            default: {
                throw new ServiceException("当前订单支付方式为" + EnumUtils.getEnumByType(payRefundDTO.getPayType()
                        , PayType.class).getInfo() + "不支持第三方APP退款");
            }
        }

    }

    @Override
    public RefundResponseDTO refundApply(PayRefundDTO payRefundDTO) {
        switch (EnumUtils.getEnumByType(payRefundDTO.getPayType(), PayType.class)) {
            case WECHAT: {
                return wechatPayService.orderRefund(payRefundDTO);
            }
            case ALIPAY: {
                return aliPayService.orderRefund(payRefundDTO);
            }
            default: {
                return RefundResponseDTO.refundError("不支持第三方APP退款");
            }
        }
    }

    /**
     * @param orderNo      订单编号
     * @param refundStatus 退款状态，枚举值：
     *                     SUCCESS：退款成功
     *                     CLOSED：退款关闭
     *                     ABNORMAL：退款异常，退款到银行发现用户的卡作废或者冻结了，导致原路退款银行卡失败，可前往【商户平台—>交易中心】，手动处理此笔退款
     */
    @Override
    public void refundNotify(String orderNo, String refundStatus) {
        String header = orderNo.substring(0, 2);
        switch (EnumUtils.getEnumByType(header, OrderNoHeaderType.class)) {
            case cloth_order_header: {
                if (OrderRefundStatus.REFUND_SUCCESS.getCode().equals(refundStatus)) {
                    orderClothService.refundSuccess(null, orderNo);
                } else if (OrderRefundStatus.REFUND_CANCEL.getCode().equals(refundStatus)) {
                    orderClothService.refundClosed(orderNo);
                } else {
                    orderClothService.refundError(orderNo);
                }
                return;
            }
            case goods_order_header: {
                if (OrderRefundStatus.REFUND_SUCCESS.getCode().equals(refundStatus)) {
                    orderGoodsService.refundSuccess(orderNo);
                } else if (OrderRefundStatus.REFUND_CANCEL.getCode().equals(refundStatus)) {
                    orderGoodsService.refundClosed(orderNo);
                } else {
                    orderGoodsService.refundError(orderNo);
                }
                return;
            }
            case recharge_order_header: {
                if (OrderRefundStatus.REFUND_SUCCESS.getCode().equals(refundStatus)) {
                    orderRechargeService.refundSuccess(orderNo);
                } else if (OrderRefundStatus.REFUND_CANCEL.getCode().equals(refundStatus)) {
                    orderRechargeService.refundClosed(orderNo);
                } else {
                    orderRechargeService.refundError(orderNo);
                }
                return;
            }
            case activity_order_header: {
                if (OrderRefundStatus.REFUND_SUCCESS.getCode().equals(refundStatus)) {
                    orderActivityService.refundSuccess(orderNo);
                } else if (OrderRefundStatus.REFUND_CANCEL.getCode().equals(refundStatus)) {
                    orderActivityService.refundClosed(orderNo);
                } else {
                    orderActivityService.refundError(orderNo);
                }
                return;
            }
        }
        throw new ServiceException("订单类型不正确");
    }

    private String wxRefund(PayRefundDTO payRefundDTO) {
        RefundService refundService = new RefundService.Builder().config(WxPayConfigs.getConfig()).build();
        CreateRequest request = new CreateRequest();
        request.setOutTradeNo(payRefundDTO.getOrderNo());
        request.setOutRefundNo(orderNoUtils.getOrderNo());
        request.setReason(payRefundDTO.getRefundReason());
        AmountReq amountReq = new AmountReq();
        amountReq.setRefund(payRefundDTO.getRefundAmount().multiply(new BigDecimal(100)).longValue());
        amountReq.setTotal(payRefundDTO.getRefundAmount().multiply(new BigDecimal(100)).longValue());
        amountReq.setCurrency("CNY");
        request.setAmount(amountReq);
        request.setNotifyUrl(WxPayConfigs.getRefundNotifyUrl());
        Refund refund = refundService.create(request);
        return OrderRefundStatus.wxStatusTransfer(refund.getStatus());
    }

    private String alipayRefund(PayRefundDTO payRefundDTO) {
        AlipayClient alipayClient = null;
        try {
            alipayClient = new DefaultAlipayClient(getAlipayConfig());
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        AlipayTradeRefundRequest request = new AlipayTradeRefundRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("out_trade_no", payRefundDTO.getOrderNo());
        bizContent.put("refund_amount", payRefundDTO.getRefundAmount().setScale(2, RoundingMode.HALF_UP).doubleValue());
        if (StringUtils.isNotEmpty(payRefundDTO.getOutRequestNo())) {
            bizContent.put("out_request_no", payRefundDTO.getOutRequestNo());
        }
        if (StringUtils.isNotEmpty(payRefundDTO.getRefundReason())) {
            bizContent.put("refund_reason", payRefundDTO.getRefundReason());
        }


        //// 返回参数选项，按需传入
        //JSONArray queryOptions = new JSONArray();
        //queryOptions.add("refund_detail_item_list");
        //bizContent.put("query_options", queryOptions);

        request.setBizContent(bizContent.toString());
        AlipayTradeRefundResponse response = null;
        try {
            response = alipayClient.certificateExecute(request);
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        if (!response.isSuccess() && !"10000".equals(response.getCode())) {
            throw new ServiceException("退款请求失败，errorCode:" + response.getCode() + ";errorMsg:"
                    + response.getMsg() + ";subCode:" + response.getSubCode() + ";subMsg:" + response.getSubMsg());
        }
        return OrderRefundStatus.REFUND_SUCCESS.getCode();
    }

    //<editor-folder desc="支付宝校验订单状态">
    @Override
    public AlipayTradeQueryResponse alipayCheckOrderStatus(String orderNo) {
        AlipayClient alipayClient = null;
        try {
            alipayClient = new DefaultAlipayClient(getAlipayConfig());
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        AlipayTradeQueryRequest request = new AlipayTradeQueryRequest();
        JSONObject bizContent = new JSONObject();
        //商户订单号，商家自定义，保持唯一性
        bizContent.put("out_trade_no", orderNo);
        request.setBizContent(bizContent.toString());
        AlipayTradeQueryResponse response = null;
        try {
            response = alipayClient.certificateExecute(request);
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        if (!response.isSuccess()) {
            log.info("支付宝支付校验状态失败，errorCode:" + response.getCode() + ";errorMsg:"
                    + response.getMsg() + ";subCode:" + response.getSubCode() + ";subMsg:" + response.getSubMsg());
        }
        return response;
    }

    //</editor-folder>

    //<editor-folder desc="付款码支付">
    @Override
    public PayResponseDTO faceToFacePay(PayOrderFaceToFaceDTO payOrderFaceToFaceDTO) {
        if (payOrderFaceToFaceDTO.getExpireTime() == null) {
            throw new ServiceException("支付缺少过期时间");
        }
        PayResponseDTO payResponseDTO = null;
        switch (EnumUtils.getEnumByType(payOrderFaceToFaceDTO.getPayType(), PayType.class)) {
            case WECHAT: {
                payResponseDTO = wechatPayService.facePay(payOrderFaceToFaceDTO);
                break;
            }
            case ALIPAY: {
                payResponseDTO = aliPayService.facePay(payOrderFaceToFaceDTO);
                break;
            }
            default:
                payResponseDTO = PayResponseDTO.payError("支付方式不能为空");
        }
        return payResponseDTO;
    }

    @Override
    public PayResponseDTO orderPayQuery(OrderPayInfo orderPayInfo) {
        PayResponseDTO payResponseDTO = null;
        switch (EnumUtils.getEnumByType(orderPayInfo.getPayType(), PayType.class)) {
            case WECHAT: {
                payResponseDTO = wechatPayService.orderPayQuery(orderPayInfo.getPayOrderNo());
                break;
            }
            case ALIPAY: {
                payResponseDTO = aliPayService.orderPayQuery(orderPayInfo.getPayOrderNo());
                break;
            }
            default:
                payResponseDTO = PayResponseDTO.payWait(orderPayInfo.getPayOrderNo());
        }
        return payResponseDTO;
    }

    @Override
    public RefundResponseDTO orderRefundQuery(PayRefundDTO payRefundDTO) {
        RefundResponseDTO refundResponseDTO = null;
        switch (EnumUtils.getEnumByType(payRefundDTO.getPayType(), PayType.class)) {
            case WECHAT: {
                refundResponseDTO = wechatPayService.orderRefundQuery(payRefundDTO.getOutRequestNo());
                break;
            }
            case ALIPAY: {
                refundResponseDTO = RefundResponseDTO.refundSuccess(payRefundDTO.getOutRequestNo(), payRefundDTO.getOutRequestNo());
                break;
            }
            default:
                refundResponseDTO = RefundResponseDTO.refundError("支付方式不能为空");
        }
        return refundResponseDTO;
    }


    private void alipayFaceToFacePay(PayOrderFaceToFaceDTO payOrderFaceToFaceDTO) {
//        Configs.init();

        DefaultAlipayClient alipayClient = null;
        try {
            alipayClient = new DefaultAlipayClient(getAlipayConfig());
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        AlipayTradePayRequest request = new AlipayTradePayRequest();
        //异步接收地址，仅支持http/https，公网可访问
        request.setNotifyUrl(AlipayConfigs.getPayNotifySuccessDomain());

        /******必传参数******/
        JSONObject bizContent = new JSONObject();
        //商户订单号，商家自定义，保持唯一性
        bizContent.put("out_trade_no", payOrderFaceToFaceDTO.getOrderNo());
        //支付金额，最小值0.01元
        bizContent.put("total_amount", payOrderFaceToFaceDTO.getPayAmount().setScale(2, RoundingMode.HALF_UP).doubleValue());
        //订单标题，不可使用特殊符号
        bizContent.put("subject", payOrderFaceToFaceDTO.getSubjectTitle());
        //1.条码场景：bar_code；2.刷脸场景：security_code
        bizContent.put("scene", "bar_code");
        //根据auth_code_type上传付款码，auth_code_type=bar_code，则填写用户支付宝钱包中的付款码；auth_code_type=security_code，则填写设备刷脸返回的付款码；
        bizContent.put("auth_code", payOrderFaceToFaceDTO.getAuthCode());

        /******可选参数******/
        //当面付场景下，当面付快捷版则传 OFFLINE_PAYMENT;其它支付宝当面付产品传 FACE_TO_FACE_PAYMENT；不传则默认使用FACE_TO_FACE_PAYMENT。
        //bizContent.put("product_code", "FACE_TO_FACE_PAYMENT");
        //// 商品明细信息
        //JSONArray goodsDetail = new JSONArray();
        //JSONObject goods1 = new JSONObject();
        //goods1.put("goods_id", "goodsNo1");
        //goods1.put("goods_name", "子商品1");
        //goods1.put("quantity", 1);
        //goods1.put("price", 0.01);
        //goodsDetail.add(goods1);
        //bizContent.put("goods_detail", goodsDetail);

        //// 扩展信息
        //JSONObject extendParams = new JSONObject();
        //extendParams.put("sys_service_provider_id", "2088511833207846");
        //bizContent.put("extend_params", extendParams);

        //// 返回参数选项
        //JSONArray queryOptions = new JSONArray();
        //queryOptions.add("fund_bill_list");
        //queryOptions.add("voucher_detail_list");
        //bizContent.put("query_options", queryOptions);

        request.setBizContent(bizContent.toString());
        AlipayTradePayResponse response = null;
        try {
            response = alipayClient.certificateExecute(request);
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        if (!response.isSuccess()) {
            throw new ServiceException("支付请求失败，errorCode:" + response.getCode() + ";errorMsg:"
                    + response.getMsg() + ";subCode:" + response.getSubCode() + ";subMsg:" + response.getSubMsg());
        }

        log.info("调用成功");

        //定时校验订单状态，如果订单已经付款完成，则终止校验，如果超过5分钟，则修改订单状态为已取消
    }

    /**
     * 支付宝付款码支付
     *
     * @param payOrderFaceToFaceDTO
     * @throws Exception
     */
    private void wxPayFaceToFace(PayOrderFaceToFaceDTO payOrderFaceToFaceDTO) {
        WxPayV2Config config = null;
        try {
            config = new WxPayV2Config();
        } catch (Exception e) {
            e.printStackTrace();
        }
        WXPay wxpay = new WXPay(config);
        Map<String, String> map = new HashMap<>(16);
        map.put("attach", "");
        map.put("auth_code", payOrderFaceToFaceDTO.getAuthCode());
        map.put("body", payOrderFaceToFaceDTO.getSubjectTitle());
        map.put("nonce_str", WXPayUtil.generateNonceStr());
        map.put("out_trade_no", payOrderFaceToFaceDTO.getOrderNo());
        map.put("spbill_create_ip", ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest().getRemoteAddr());
        map.put("total_fee", payOrderFaceToFaceDTO.getPayAmount().multiply(new BigDecimal(100)).intValue() + "");
        //生成签名
        String sign = null;
        try {
            sign = WXPayUtil.generateSignature(map, config.getKey());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException("微信签名生成失败");
        }
        map.put("sign", sign);
        String mapToXml = null;
        try {
            //调用微信的扫码支付接口
            Map<String, String> resp = wxpay.microPay(map);
            mapToXml = WXPayUtil.mapToXml(resp);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("微信支付失败" + e);
        }
        //判断支付是否成功
        String return_code = null;
        String return_msg = null;
        String result_code = null;
        String err_code_des = null;
        String err_code = null;
        //获取Document对象（主要是获取支付接口的返回信息）
        Document doc = null;
        try {
            doc = DocumentHelper.parseText(mapToXml);
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        //获取对象的根节点<xml>
        Element rootElement = doc.getRootElement();
        //获取对象的子节点
        List<Element> elements = rootElement.elements();
        for (Element element : elements) {
            if (element.getName().equals("return_code")) {
                return_code = element.getTextTrim();
            } else if (element.getName().equals("return_msg")) {
                return_msg = element.getTextTrim();
            } else if (element.getName().equals("result_code")) {
                result_code = element.getTextTrim();
            } else if (element.getName().equals("err_code_des")) {
                err_code_des = element.getTextTrim();
            } else if (element.getName().equals("err_code")) {
                err_code = element.getTextTrim();
            }
        }
        if (!WxPayV2Config.PAY_SUCCESS.equals(return_code)) {
            throw new ServiceException("付款码支付失败，return_msg: " + return_msg);
        }
    }

    //</editor-folder>

    private AlipayConfig getAlipayConfig() {
        AlipayConfig alipayConfig = new AlipayConfig();
        //设置网关地址
//        alipayConfig.setServerUrl(Configs.getOpenApiDomain());
        //设置应用APPID
        alipayConfig.setAppId(AlipayConfigs.getAppid());
        //设置应用私钥
        alipayConfig.setPrivateKey(AlipayConfigs.getPrivateKey());
        //设置应用公钥证书路径
        alipayConfig.setAppCertPath(AlipayConfigs.getPublicKey());
        //设置支付宝公钥证书路径
        alipayConfig.setAlipayPublicCertPath(AlipayConfigs.getAlipayPublicKey());
        //设置支付宝根证书路径
        alipayConfig.setRootCertPath(AlipayConfigs.getAlipayRootKey());
        //设置请求格式，固定值json
//        alipayConfig.setFormat("json");
        //设置字符集
//        alipayConfig.setCharset("UTF-8");
        //设置签名类型
        alipayConfig.setSignType(AlipayConfigs.getSignType());

        return alipayConfig;
    }

    //<editor-folder desc="app支付相关接口">
    private AppPayRespVO wxXcxPay(PayOrderAppDTO payOrderAppDTO) {
        // 构建service
        JsapiService service = new JsapiService.Builder().config(WxPayConfigs.getConfig()).build();
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest request = new PrepayRequest();
        com.wechat.pay.java.service.payments.jsapi.model.Amount amount = new com.wechat.pay.java.service.payments.jsapi.model.Amount();
        amount.setTotal(payOrderAppDTO.getPayAmount().multiply(new BigDecimal(100)).intValue());
        request.setAmount(amount);
        request.setAppid(wxMaProperties.getAppid());
        request.setMchid(WxPayConfigs.getMch_ID());
        request.setDescription(payOrderAppDTO.getSubjectTitle());
        request.setNotifyUrl(WxPayConfigs.getPayNotifyUrl());
        request.setOutTradeNo(payOrderAppDTO.getOrderNo());
        Payer payer = new Payer();
        payer.setOpenid(AuthHolder.openId());
        request.setPayer(payer);
        // 调用下单方法，得到应答
        com.wechat.pay.java.service.payments.jsapi.model.PrepayResponse response = service.prepay(request);

        return new AppPayRespVO(response.getPrepayId());
    }

    private AppPayRespVO wxAppPay(PayOrderAppDTO payOrderAppDTO) {
        // 构建service
        AppServiceExtension service = new AppServiceExtension.Builder().config(WxPayConfigs.getConfig()).build();
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        com.wechat.pay.java.service.payments.app.model.PrepayRequest request = new com.wechat.pay.java.service.payments.app.model.PrepayRequest();
        com.wechat.pay.java.service.payments.app.model.Amount amount = new com.wechat.pay.java.service.payments.app.model.Amount();
        amount.setTotal(payOrderAppDTO.getPayAmount().multiply(new BigDecimal(100)).intValue());
        request.setAmount(amount);
        request.setAppid(WxPayConfigs.getAppId());
        request.setMchid(WxPayConfigs.getMch_ID());
        request.setDescription(payOrderAppDTO.getSubjectTitle());
        request.setNotifyUrl(WxPayConfigs.getPayNotifyUrl());
        request.setOutTradeNo(payOrderAppDTO.getOrderNo());
        // 调用下单方法，得到应答
        PrepayWithRequestPaymentResponse response = service.prepayWithRequestPayment(request);
        return new AppPayRespVO(response);
    }

    private AppPayRespVO alipayAppPay(PayOrderAppDTO payOrderAppDTO) {
        if (payOrderAppDTO.getExpireTime() == null) {
            throw new ServiceException("支付缺少过期时间");
        }
        AlipayClient alipayClient = null;
        try {
            alipayClient = new DefaultAlipayClient(getAlipayConfig());
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
        request.setNotifyUrl(AlipayConfigs.getPayNotifySuccessDomain());
        JSONObject bizContent = new JSONObject();
        bizContent.put("out_trade_no", payOrderAppDTO.getOrderNo());
        bizContent.put("total_amount", payOrderAppDTO.getPayAmount().setScale(2, RoundingMode.HALF_UP).doubleValue());
        bizContent.put("subject", payOrderAppDTO.getSubjectTitle());
        bizContent.put("product_code", "QUICK_MSECURITY_PAY");
        if (payOrderAppDTO.getExpireTime() != null) {
            bizContent.put("time_expire", DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, payOrderAppDTO.getExpireTime()));
        }
        request.setBizContent(bizContent.toString());
        AlipayTradeAppPayResponse response = null;
        try {
            response = alipayClient.sdkExecute(request);
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }

        if (!response.isSuccess()) {
            throw new ServiceException("支付请求失败，errorCode:" + response.getCode() + ";errorMsg:"
                    + response.getMsg() + ";subCode:" + response.getSubCode() + ";subMsg:" + response.getSubMsg());
        }

        log.info("调用成功");
        String orderStr = response.getBody();

        //定时校验订单状态，如果订单已经付款完成，则终止校验，如果超过5分钟，则修改订单状态为已取消
        return new AppPayRespVO(orderStr);
    }
    //</editor-folder>


}
